
uniform float shade;

vec4 normalizeColor(vec4 color)
{
    float m = max(max(color.r, color.g), max(color.b, 0.01));
    return vec4(color.rgb / m, color.a);
}

vec4 partiallyNormalizeColor(vec4 color, float normalizeFactor)
{
    vec4 normColor = normalizeColor(color);
    return normalizeFactor * normColor + (1.0 - normalizeFactor) * color;
}

vec4 cmy2rgb(vec4 cmy)
{
    vec4 cyan = vec4(0,1,1,1);
    vec4 magenta = vec4(1,0,1,1);
    vec4 yellow = vec4(1,1,0,1);
    
    vec4 cmyShaded = cmy + ( 0.9 + 0.1 * cmy.w ) * shade * ( 1.0 - cmy );
    
    vec4 c = (1.0 - cmyShaded.x * ( 1.0 - cyan ));
    vec4 m = (1.0 - cmyShaded.y * ( 1.0 - magenta ));
    vec4 y = (1.0 - cmyShaded.z * ( 1.0 - yellow ));
    vec4 rgbColor = (c * m * y);
    return vec4(rgbColor.rgb, 1.0);
}

void main()
{
    gl_FragColor = cmy2rgb( gl_Color );
}
